/************************************************************************
 ========================================================================
 CORAL 
 (c)  Copyright R. Ramakrishnan and The CORAL Group, 
 University of Wisconsin at Madison.
 (1992) All Rights Reserved.
 Version 0.1
 ========================================================================



 ------------------------------------------------------------------------
 CORAL Version 0.1
 RESEARCH SOFTWARE DISCLAIMER -------------------------------------------
 ------------------------------------------------------------------------

    As unestablished, research software, this program is provided free of 
    charge on an "as is" basis without warranty of any kind, either 
    express or implied.  Acceptance and use of this program constitutes 
    the user's understanding that (s)he will have no recourse for any 
    actual or consequential damages, including, but not limited to, 
    lost profits or savings, arising out of the use of or inability to 
    use this program.  

 ------------------------------------------------------------------------
 USER AGREEMENT ---------------------------------------------------------
 ------------------------------------------------------------------------

     BY ACCEPTANCE AND USE OF THIS EXPERIMENTAL PROGRAM
     THE USER AGREES TO THE FOLLOWING:

     a.  This program is provided free of charge for the user's personal, 
	 non-commercial, experimental use.

     b.  All title, ownership and rights to this program and any copies 
         remain with the copyright holder, irrespective of the ownership 
	 of the media on which the program resides.

     c.  The user is permitted to create derivative works to this program.  
         However, all copies of the program and its derivative works must
         contain the CORAL copyright notice, the UNESTABLISHED SOFTWARE 
         DISCLAIMER and this USER AGREEMENT.

     d.  The user understands and agrees that this program and any 
         derivative works are to be used solely for experimental purposes 
	 and are not to be sold or commercially exploited in any manner 
	 WITHOUT EXPRESS WRITTEN PERMISSION.

     e.  We request that the user supply us with a copy of any changes, 
         enhancements, or derivative works which the user may create,
	 with the user's permission to redistribute it.
	 Copies of such material should be sent to:  CORAL@CS.WISC.EDU

-------------------------------------------------------------------------
*************************************************************************/

/***********************************************************************
	CORAL Software :: U.W.Madison

	hashtable.h : Header file

	Contains the following class/type  declarations	 :

 ***********************************************************************/
#ifndef CORAL_HASHTABLE_H
#define CORAL_HASHTABLE_H

typedef int (*EqualsFn)(void*,void*);

/* 
 * this must remain a struct and not a class. this is because bzero is
 * used immediately after allocating an array of these, and if there are
 * any methods, the method table will be overwritten
 */
typedef struct HashEntry {
	void *data;
	int hashval;
	struct HashEntry *next;
      } HashEntry;


class GenericHashTable{
    
	public:

	EqualsFn equalsfn;
	int max_size;
	double incr_ratio;
	double max_occupancy;
	int cur_size;
	int limit;
	 
	GenericHashTable(EqualsFn eqf1, int initsize1=128, 
		double incr_ratio1 = 1.5,
		double max_occupancy1 = 0.7);
	virtual void *insert(void *entry, int hashval, int& to_delete)=0;
	virtual void *lookup(void *entry, int hashval)=0;
};

class CoreHashTable: public GenericHashTable{

	HashEntry *table;

	public:

	CoreHashTable(EqualsFn eqf1, int initsize1=128, 
		double incr_ratio1 = -1.0, double max_occupancy1 = -1.0);
	inline void rehash_if_needed() {
		if (cur_size > limit) rehash_bigger();
		}

	virtual void *insert(void *entry, int hashval, int& to_delete);
	void *insert(void *entry, int hashval) {
	   int to_delete = 0;
	   return insert(entry, hashval, to_delete); 
	}
	virtual void *lookup(void *entry, int hashval);
	void rehash_bigger();
};

#endif /* !CORAL_HASHTABLE_H */

